/* Kb.java
** A simple class for command line IO in Java.
**
** This class is not instantiable, and attaches itself statically to
** System.in when its methods are called.  The public methods are:
**
** public static String getLine()
** public static String getLine(String prompt)
** - returns the next line from System.in.  The version with the prompt
**   parameter prints the specified prompt to the console.  The prompt-less
**   version is mostly for internal use. 
**
** public static String getWord()
** public static String getWord(String prompt)
** - returns the next word from System.in.  The version with the prompt
**   parameter prints the specified prompt to the console.  The prompt-less
**   version is mostly for internal use. 
**
** public static int getInt(String prompt, int min, int max)
** public static int getInt(String prompt)
** - prints out the prompt and returns an int in the range [min,max]m if
**   a range is specified
**
** public static double getDouble(String prompt, double min, double max)
** public static double getDouble(String prompt)
** - prints out the prompt and returns a double in the range [min,max]m if
**   a range is specified
**
** Both getInt() and getDouble() have validation loops the persist until 
** valid input is received.
**
**
** This file is a modification of an assignment from CS51 at the University
** of San Diego.  Parts of this code were written by John Glick, Lukasz
** Pruski, and Dwight Bean.
*/

        
import java.io.*;
import java.util.*;

public class Kb
{
      private static StringTokenizer tokens = null;
      private static BufferedReader bufferedReader;
      static  // initialization of static data bufferedReader.
      { 
	 InputStreamReader  instre = new InputStreamReader(System.in);
	 bufferedReader = new BufferedReader(instre);
      }

   
      // this class is not instantiable
      private Kb()
      {  // should never be called
      }
   
  
      // called when tokens is empty.  Read another line. Exit on end-of-input.
      private static void readLine()
      {
	 try
	 {
	    String line = bufferedReader.readLine();
	    tokens = new StringTokenizer(line);
	 }
	 catch (IOException ioe)
	 {
	    System.out.println("Kb Error: readLine()."
			       + "  Program termination.");
	    System.exit(1);
	 }
      }

   
      // returns next line entered; exception if end-of-input
      public static String getLine()
      {
	 String line = "";
	 try
	 {
	    line = bufferedReader.readLine();
	 }
	 catch (IOException ioe)
	 {
	    System.out.println("Kb Error: readLine()."
			       + "  Program termination.");
	    System.exit(1);
	 }
	 return line;
      }
   
      // prompts with p; returns next line from input
      // exception if end-of-input
      public static String getLine(String p)
      {
	 System.out.print(p);
	 return getLine();      
      }
   
   
      // returns the next word entered at the keyboard.
      // throws NoSuchElementException and exits on end-of-input
      // echoes input before return if p is "".
      public static String getWord()
      {  
	 // Make sure tokens are available.
	 while (tokens == null || !tokens.hasMoreTokens())
	    readLine();
	 // Get the string and return it.
	 try
	 {
	    return tokens.nextToken();
	 }
	 catch (NoSuchElementException e)
	 {
	    System.out.println("Kb Error: getWord()."
			       + "  Program terminating.");
	    System.exit(1);
	    return null;
	 }
      }
   
      // Prompts with p and returns the next word entered at the keyboard.
      // throws NoSuchElementException and exits on end-of-input
      // echoes input before return if p is "".
      public static String getWord(String p)
      {
	 System.out.print(p);
	 return getWord();
      }

   
      // Prompts with p; returns a "valid" integer in [min,max] using
      // validation loop
      public static int getInt(String p, int min, int max)
      {
	 System.out.print(p);
	 boolean done = false; 
	 int n = 0; 
	 while(!done)  // validation loop to ensure valid input
	 {
	    String s = getWord();
	    try
	    {
	       n = Integer.parseInt(s);
	       if (n < min || n > max)  
		  System.out.print("Enter int in [" 
				   + min + "," + max + "]: ");
	       else 
		  done = true;               
	    }        
	    catch(NumberFormatException e)
	    { 
	       System.out.print("Enter valid int: ");
	    }
	 }
	 return n;
      }

      // Prompts with p; returns a "valid" integer using validation loop
      public static int getInt(String p)
      {
	 return getInt(p,Integer.MIN_VALUE,Integer.MAX_VALUE);
      }


      // Prompts with p; returns a "valid" double in [min,max] using
      // validation loop
      public static double getDouble(String p, double min, double max)
      {
	 System.out.print(p);
	 double n = 0.0;  
	 boolean done = false; 
	 while (!done)  // loop until valid input is received
	 {
	    String s = getWord();
	    try
	    {
	       n = Double.valueOf(s).doubleValue();
	       if (n < min || n > max)  
		  System.out.print("Enter double in [" 
				   + min + "," + max + "]: ");
	       else 
		  done = true;               
	    }
	    catch(NumberFormatException e)
	    {  
	       System.out.print("Enter valid double: ");
	    }
	 }
	 return n;
      }

      // Prompts with p; returns a "valid" double using validation loop
      // or the program terminates with an error message if p is "".
      // Echoes input before return if p is "".
      public static double getDouble(String p)
      {
	 System.out.print(p);
	 return getDouble("",Double.NEGATIVE_INFINITY,
			  Double.POSITIVE_INFINITY);
      }

}


