/* ParrondoApplet.java is a graphic interface for Parrondo.java.  It's
 * a bit of a hack, as I'm not really a Java programmer.
 * author - Asher Langton (langton@cs.wisc.edu)
 * version 2.01, 10-12-02 (updated to fix a thread issue)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;

public class ParrondoApplet extends Applet implements ActionListener, Runnable
{
      private GraphCanvas graphs;  // displays graphs 
      private KeyCanvas key;       // displays graph key 
      private TextGridPanel text;  // holds all four editable Textfields
      private Button runButton;
      private Thread thr;          // threads the calculation process so 
      // the "Calculting . . ." animation
      // can be run in parallel 
   
      private boolean curThread;   // true iff there is currently a calculation
      // thread launched by actionPreformed().  This
      // flag will allow us to avoid launching
      // multiple (conflicting) threads
   
      public void init()
      {
	 setSize(550,500);             //overall applet size
	 setBackground(Color.white);
	 graphs = new GraphCanvas();
	 add(graphs);
	 key = new KeyCanvas();
	 add(key);
	 text = new TextGridPanel();
	 add(text);
	 runButton = new Button("        Run        ");
	 runButton.addActionListener(this);  // all actions come from runButton
	 add(runButton);
      }
   
      // run() performs the gets values from the TextFields, starts the 
      // "calculating" animation, performs the calculations, then stops 
      // the animation
      public void run()
      {
	 showStatus("Calculating . . .");
	 graphs.setInitialized(false);
	 graphs.repaint(); // should clear the screen; doesn't on Netscape 4.6
	 int trials = text.getTrlValue();    // get user-inputs
	 double eps = text.getEpsValue();
	 int mod = text.getModValue();
	 int per = 2 * text.getAltValue();
	 graphs.startWaiting();              // start animation thread
	 Parrondo p = new Parrondo(trials, eps, mod, per); //calculate
	 graphs.setaOnly(p.getaOnly());      // give results to GraphCanvas
	 graphs.setbOnly(p.getbOnly());
	 graphs.setabAlt(p.getabAlt());
	 graphs.setabRand(p.getabRand());
	 graphs.stopWaiting();               // end animation threa
	 graphs.setInitialized(true);
	 graphs.repaint();                   // paint new graphs
	 showStatus("Done.");
	 curThread = false;  // this thread's done, so we're free to launch 
	 // a new one
      }
   
      // called when runButton is pushed; creates new Thread and executes run()
      public synchronized void actionPerformed(ActionEvent e)
      {
	 if (!curThread) //launch new thread iff there isn't already one active
	 {
	    curThread = true;
	    thr = new Thread(this);
	    thr.start();
	 }
      }
}   



/* This canvas shows the 4 graphs */
class GraphCanvas extends Canvas implements Runnable
{
      private Thread thr;                //animation thread
      private double[] aOnly, bOnly, abAlt, abRand;  //data arrays
      private boolean initialized;
      private volatile boolean waiting;   // flags for paint()
      private Image im;         // image for double-buffering
      private Graphics gImage;  // im's Graphics
      private String s;         // the animated String
   
      public static final int POINTS = 101; //size of data arrays

      // constructor
      public GraphCanvas()
      {
	 setSize(550, 350);
	 initialized = false;
	 waiting = false;
	 setBackground(Color.white);
	 s = "";
      }
   
      /* paints the graph axes; if initialized==true, the graphs themselves are
      ** painted; otherwise, if waiting is true, the current value of 's' is
      ** painted; double buffering is used to prevent flickering */
      public void paint(Graphics g)
      {
	 im = createImage(550,350);      
	 gImage = im.getGraphics();
	 gImage.setColor(Color.black);
	 gImage.drawLine(25,25,25,325);     //draw axes
	 gImage.drawLine(25,175,525,175);
	 gImage.setColor(Color.lightGray);
	 gImage.drawLine(26,25,525,25);     // draw background lines
	 gImage.drawLine(26,75,525,75);
	 gImage.drawLine(26,125,525,125);
	 gImage.drawLine(26,225,525,225);
	 gImage.drawLine(26,275,525,275);
	 gImage.drawLine(26,325,525,325);
	 gImage.drawLine(525,25,525,325);
	 if (waiting)                      
	 {
	    gImage.setColor(Color.red);     // animation
	    gImage.setFont(new Font("Bold15", Font.BOLD, 15));
	    gImage.drawString(s, 225, 100);
	 }
	 else if (initialized)
	    plotPoints(gImage);             // plot graphs
	 gImage.setColor(Color.black);      // paint labels 
	 gImage.setFont(new Font("Plain12", Font.PLAIN, 12));
	 gImage.drawString("+3",7,35);
	 gImage.drawString("-3",7,325);
	 gImage.drawString("100",525,173);
	 gImage.setFont(new Font("Bold12", Font.BOLD, 12));
	 gImage.drawString("Capital",28,25);
	 gImage.drawString("Games",483,189);
	 g.drawImage(im,0,0,this);          // paint image to screen
      }
   
      // starts animation thread
      public void startWaiting()
      {
	 waiting = true; // now 's' will be painted
	 thr = new Thread(this);
	 thr.start();
      }
   
      // kills animation thread
      public void stopWaiting()
      {
	 waiting = false; // now 's' will not be painted
      }
   
      // plots all four arrays, in different colors; array size is already 
      // confirmed
      public void plotPoints(Graphics g)
      {
	 int x2, y2;
	 g.setColor(Color.red);
	 int x1 = 25, y1 = 175;
	 for (int i = 1; i < POINTS; i++)
	 {
	    x2 = 25 + (i * 5);
	    y2 = 175 - (int)Math.round(aOnly[i] * 50);
	    g.drawLine(x1,y1,x2,y2);
	    x1 = x2; 
	    y1 = y2;
	 }
	 g.setColor(Color.green);
	 x1 = 25; y1 = 175;
	 for (int i = 1; i < POINTS; i++)
	 {
	    x2 = 25 + (i * 5);
	    y2 = 175 - (int)Math.round(bOnly[i] * 50);
	    g.drawLine(x1,y1,x2,y2);
	    x1 = x2; 
	    y1 = y2;
	 }
	 g.setColor(Color.blue);
	 x1 = 25; y1 = 175;
	 for (int i = 1; i < POINTS; i++)
	 {
	    x2 = 25 + (i * 5);
	    y2 = 175 - (int)Math.round(abAlt[i] * 50);
	    g.drawLine(x1,y1,x2,y2);
	    x1 = x2; 
	    y1 = y2;
	 }
	 g.setColor(Color.cyan);
	 x1 = 25; y1 = 175;
	 for (int i = 1; i < POINTS; i++)
	 {
	    x2 = 25 + (i * 5);
	    y2 = 175 - (int)Math.round(abRand[i] * 50);
	    g.drawLine(x1,y1,x2,y2);
	    x1 = x2; 
	    y1 = y2;
	 }
      }
      
      // the animation thread.  changes the "Calculating  . . ." string and 
      // repaints it as long as the calculation thread is running
      public void run()
      {
	 try
	 {
	    while(waiting) // waiting is modified by a different thread
	    {
	       s = "Calculating";  repaint();  Thread.sleep(500);
	       s = s + " .";  repaint();  Thread.sleep(500);
	       s = s + " .";  repaint();  Thread.sleep(500);
	       s = s + " .";  repaint();  Thread.sleep(500);
	    } 
	 }
	 catch (InterruptedException e)
	 {
	 }
      }
   
      // overrides update for smoother animation
      public void update(Graphics g)
      {
	 paint(g);
      }
   
      /* the following four methods get the Parrondo data and check the array 
      ** sizes */
      public void setaOnly(double[] d) throws IllegalArgumentException
      {
	 if (d.length == POINTS)
	    aOnly = d;
	 else
	    throw new IllegalArgumentException("Bad array size!");
      }
   
      public void setbOnly(double[] d) throws IllegalArgumentException
      {
	 if (d.length == POINTS)
	    bOnly = d;
	 else
	    throw new IllegalArgumentException("Bad array size!");
      }
   
      public void setabAlt(double[] d) throws IllegalArgumentException
      {
	 if (d.length == POINTS)
	    abAlt = d;
	 else
	    throw new IllegalArgumentException("Bad array size!");
      }
   
      public void setabRand(double[] d) throws IllegalArgumentException
      {
	 if (d.length == POINTS)
	    abRand = d;
	 else
	    throw new IllegalArgumentException("Bad array size!");
      }
   
      // allows initialized flag to be toggled externally
      public void setInitialized(boolean b)
      {
	 initialized = b;
      }
   
}


// This canvas displays the color-coded key
class KeyCanvas extends Canvas
{
      public KeyCanvas()
      {
	 setSize(200,100);
      }
   
      // paints the key; draws four colored boxes with corresponding labels
      public void paint(Graphics g)
      {
	 g.setColor(Color.red);
	 g.fillRect(25,0,25,25);
	 g.setColor(Color.green);
	 g.fillRect(25,25,25,25);
	 g.setColor(Color.blue);
	 g.fillRect(25,50,25,25);
	 g.setColor(Color.cyan);
	 g.fillRect(25,75,25,25);
	 g.setColor(Color.black);
	 g.drawString("Game A only",51,20);
	 g.drawString("Game B only",51,45);
	 g.drawString("A & B, alternating",51,70);
	 g.drawString("A & B, randomly",51,95);
      }
}

// holds all of the editable textfields, which are objects of type TextPanel 
// (see below)
class TextGridPanel extends Panel
{
      private IntTextPanel mod, alt, trl; // modulus, alternation, and trials 
      // TextPanels
      private DoubleTextPanel eps;        // epsilon TextPanel 
   
      public TextGridPanel()
      {
	 setSize(350,100);
	 setLayout(new GridLayout(2,2));
	 mod = new IntTextPanel("Modulus (2-5)","3",2,5);//construct TextPanels
	 eps = new DoubleTextPanel("Epsilon (0.0-0.095)",".005",0.0,.095);
	 alt = new IntTextPanel("Alternation (1-50)","2",1,50);
	 trl = new IntTextPanel("Trials (1-50000)","1000",1,50000);
	 add(mod);
	 add(eps);
	 add(alt);
	 add(trl);
      }
   
      /* Accessor methods to get user input; verification is done at the 
      ** TextPanel level */
      public int getModValue()
      {
	 return mod.getValue();
      }
   
      public int getAltValue()
      {
	 return alt.getValue();
      }
   
      public int getTrlValue()
      {
	 return trl.getValue();
      }
   
      public double getEpsValue()
      {
	 return eps.getValue();
      }
}

/* A simple Panel consisting of two TextFields: a label, and an editable 
** integer field */
class IntTextPanel extends Panel
{
      private int min, max; // min & max values for user input
      private TextField txt,lbl; 

      public IntTextPanel(String label, String txtstring, int mn, int mx)
      {
	 setLayout(new GridLayout(2,1));
	 lbl = new TextField(label, 16);
	 lbl.setBackground(Color.gray);  // make sure label is distinguishable 
	 // from editable text 
	 lbl.setForeground(Color.black);
	 lbl.setEditable(false);
	 add(lbl);
	 txt = new TextField(txtstring,5);
	 txt.setBackground(Color.white);
	 txt.setForeground(Color.black);
	 add(txt);
	 min = mn;
	 max = mx;
      }
   
      // returns current value in txt; if too high or low, the value is set to 
      // max or min and that value is returned
      public int getValue()
      {
	 int temp = Integer.parseInt(txt.getText());
	 if (temp > max)
	 {
	    temp = max;
	    txt.setText(max + "");
	 }
	 else if (temp < min)
	 {
	    temp = min;
	    txt.setText(min + "");
	 }
	 return temp;
      }
}

/* identical to IntTextPanel, but handles doubles */
class DoubleTextPanel extends Panel
{
      private double min, max;  // min & max values for user input
      private TextField txt,lbl;

      public DoubleTextPanel(String label, 
			     String txtstring, 
			     double mn, 
			     double mx)
      {
	 setLayout(new GridLayout(2,1));
	 lbl = new TextField(label, 16);
	 lbl.setBackground(Color.gray); // make sure label is distinguishable 
	 // from editable text 
	 lbl.setForeground(Color.black);
	 lbl.setEditable(false);
	 add(lbl);
	 txt = new TextField(txtstring,5);
	 txt.setBackground(Color.white);
	 txt.setForeground(Color.black);
	 add(txt);
	 min = mn;
	 max = mx;
      }
   
      // returns current value in txt; if too high or low, the value is set to 
      //max or min and that value is returned
      public double getValue()
      {
	 double temp = Double.valueOf(txt.getText()).doubleValue(); 
	 // parseDouble causes portability problems
	 if (temp > max)
	 {
	    temp = max;
	    txt.setText(max + "");
	 }
	 else if (temp < min)
	 {
	    temp = min;
	    txt.setText(min + "");
	 }
	 return temp;
      }
}
